/*
 * mcountd.c - millennium count down
 * version 0.4
 *
 *  Copyright (c) by 	Karl-Martin Skontorp <kms@users.sourceforge.net>
 *  			Martin Dahl <dahlm@vf.telia.no>
 *                   	Anders Hermansen <ahermans@vf.telia.no>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <time.h>
#include <unistd.h>

#include "mcountd.h"

int main(int argc, char *argv[]);
void print_number(char *nmbr);
void show_params();
void show_version();

int main(int argc, char **argv) 
{
    long targettime = DEFAULT_TARGETTIME;
    int updaterate = DEFAULT_UPDATERATE;

    long curtime;
    long difftime = 0;
    long last_time_printed;
    int rollover = 0;
    int enable_rollover = 1;
    int exit_on_rollover = 0;

    int dayss;
    int hours;
    int mins;
    int secs;

    int filler_lines;

    char timeleft[ 128 ];

    int a;

    /* Parse the command line */
    if (argc!= 1)
    {
	    for (a=1; a<argc; a++)
	    {
		    if (argv[a][0]!='-') 
			    show_params(); 
		    else /* -* */
		    {
			    if (argv[a][1]=='t') /* -t targettime */
			    {
				    a++;
				    targettime = atol(argv[a]);
			    } else
			    {
				    if (argv[a][1]=='u') /* -u updaterate */
				    {
					    a++;
					    updaterate = atol(argv[a]);
				    } else
				    {
					    if (argv[a][1]=='h') /* -h */
					    {
						    show_params();
					    } else 
					    {
						    if (argv[a][1]=='v') /* -v */
						    {
							    show_version();
						    } else 
						    {
							    if (argv[a][1]=='r') /* -r */
							    {
								    enable_rollover = 0;
							    } else 
							    {
								    if (argv[a][1]=='e') /* -e */
								    {
									    exit_on_rollover = 1;
								    } else 
								    {
									    if (argv[a][1]!=104)
									    {
										    printf("Unknown parameter.\n");
										    show_params();
									    }
								    }
							    }
						    }
					    }
				    }
			    }
		    }
	    }
    }

    last_time_printed = -1;

    while (1) 
    {
	    /* Compute times */
	    curtime = time(0); 

	    if ((last_time_printed+updaterate-1) < curtime)
	    {
		    difftime = abs(targettime - curtime);
		    
		    if (targettime < curtime)
		    {
			    rollover = 1;

			    if (!enable_rollover)
			    {
				    difftime = 0;
			    }
		    }

		    dayss=difftime/86400;
		    hours=(difftime%86400)/3600;
		    mins=(difftime%3600)/60;
		    secs=difftime%60;

		    if (rollover) 
		    {
			    if (exit_on_rollover) {
				    exit(0);
			    }
			    printf("%i days since roll-over.\n\n", dayss);
		    } else
		    {
			    printf("%i days left.\n\n", dayss);
		    }

		    sprintf(timeleft, "%2.2i:%2.2i:%2.2i", hours,mins,secs);

		    print_number( timeleft );
		    filler_lines = TLINES - (CHARACTER_IMAGE_HEIGHT + 2) -1;
		    while (filler_lines--)
			    fputc('\n', stdout);
		    last_time_printed = curtime;

	    }

	    usleep(250);
    }
    return 0;
}

void print_number(char *nmbr )
{
	unsigned y;
	int c;
	char * nmbr_index,
	** image_source;

	for( y = 0; y < CHARACTER_IMAGE_HEIGHT; y++ )
	{
		for( nmbr_index = nmbr; ( c = *nmbr_index++ ); )
		{
			image_source = isdigit( c )
				? number_pointer[ c - '0' ]
				: colon_pointer;
			printf( "%s ", image_source[ y ] );
		}
		fputc( '\n', stdout );
	}
}

void show_version()
{
	printf("mcountd - millennium count down, version 0.4\n");
	printf("This is free software with ABSOLUTELY NO WARRANTY.\n");
	printf("See included file COPYING for more information.\n");
	exit(1);
}

void show_params() 
{
	printf("mcountd - 	millennium count down, version 0.4\n");
	printf("		by Karl-Martin Skontorp,\n");
	printf("		Martin Dahl and Anders Hermansen.\n");
	printf("		http://mcountd.sourceforge.net\n");
	printf("\n");
	printf("This is free software with ABSOLUTELY NO WARRANTY.\n");
	printf("See included file COPYING for more information.\n");
	printf("\n");
	printf("options:	-t	Targettime for count down in seconds since\n");
	printf("			01.01.1970 00:00:00. Default: 01.01.2000 00:00:00.\n");
	printf("		-u	Seconds between update. Default: 1.\n");
	printf("		-r	Disable roll-over. Default: enabled.\n");
	printf("		-e	Exit on roll-over. Default: disabled.\n");
	printf("		-v	Show version.\n");
	printf("		-h	Show this message.\n");
	exit(1);
}
